@@ -1,6 +1,6 @@
 /* Implementation of the Applet class.
  *
- * Copyright (c) 2003, 04 Ole Laursen.
+ * Copyright (c) 2003, 04, 05 Ole Laursen.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -339,9 +339,18 @@ void Applet::viewer_type_listener(unsigned int, Gnome::Conf::Entry entry)
       set_view(new CurveView);
   }
   else if (s == "bar") {
-    if (!dynamic_cast<BarView *>(view.get()))
+    // It get's tricky here because them BarView can render 2 viewers.
+    // Thus, we much also check the oriententation
+    BarView *bar_view = dynamic_cast<BarView *>(view.get());
+    if (!(bar_view && bar_view->is_horizontal()) )
       set_view(new BarView);
   }
+  else if (s == "vbar") {
+    // Same situation as with "bar"
+    BarView *bar_view = dynamic_cast<BarView *>(view.get());
+    if (!(bar_view && !bar_view->is_horizontal()) )
+      set_view(new BarView(false));
+  }
   else if (s == "text") {
     if (!dynamic_cast<TextView *>(view.get()))
       set_view(new TextView);
@@ -451,22 +460,25 @@ void Applet::on_about_activated()
   std::vector<Glib::ustring> documenters;
   // add documenters here
 
-  // note to translators: please fill in your names and email addresses
-  Glib::ustring translators = _("translator-credits");
-
   Glib::ustring description =
     _("Monitor various hardware-related information, such as CPU usage, "
-      "memory usage etc. Supports curve graphs, horizontal bar plots, "
+      "memory usage etc. Supports curve graphs, bar plots, "
       "column diagrams, textual monitoring and fluctuating flames.");
   
   if (about.get() == 0) {
-    about.reset(new Gnome::UI::About(_("Hardware Monitor"), VERSION,
-				     // %1 is the copyright symbol
-				     String::ucompose(_("Copyright %1 2003 Ole Laursen"), "\xc2\xa9"),
-				     authors, documenters, description,
-				     translators == "translator-credits"
-				     ? "" : translators,
-				     icon));
+    about.reset(new Gtk::AboutDialog());
+    about->set_name(_("Hardware Monitor"));
+    about->set_version(VERSION);
+    // %1 is the copyright symbol
+    about->set_copyright(String::ucompose(_("Copyright %1 2003 Ole Laursen"),
+					  "\xc2\xa9"));
+    about->set_authors(authors);
+    if (!documenters.empty())
+      about->set_documenters(documenters);
+    about->set_comments(description);
+    // note to translators: please fill in your names and email addresses
+    about->set_translator_credits(_("translator-credits"));
+    about->set_logo(icon);
     about->set_icon(icon);
     about->show();
   }
