@@ -37,7 +37,7 @@
 class Bar
 {
 public:
-  Bar(Monitor *monitor);
+  Bar(Monitor *monitor, bool horizontal = false);
   ~Bar();
 
   void update();
@@ -52,11 +52,14 @@ private:
   box_sequence boxes;
 
   double old_value, new_value;
+  bool horizontal;
 };
 
-Bar::Bar(Monitor *m)
+Bar::Bar(Monitor *m, bool horizontal)
   : monitor(m), old_value(0), new_value(0)
-{}
+{
+  this->horizontal = horizontal;
+}
 
 Bar::~Bar()
 {
@@ -116,12 +119,27 @@ void Bar::draw(Gnome::Canvas::Canvas &canvas,
   unsigned int outline_color = outlineified(fill_color);
   
   // calculate parameters
-  int const box_width = 3, min_spacing = 2;
+  int box_size;
   // use min_spacing at least, except for last box which doesn't need spacing
-  int total_no_boxes = (width + min_spacing) / (box_width + min_spacing);
-  double box_spacing
-    = (double(width) - box_width * total_no_boxes) / (total_no_boxes - 1);
+  int total_no_boxes;
+  double box_spacing;
 
+  if (this->horizontal) {
+    box_size = 3;
+    int const min_spacing = 2;
+    total_no_boxes = (width + min_spacing) / (box_size + min_spacing);
+    box_spacing = (double(width) - box_size * total_no_boxes) / (total_no_boxes - 1);
+  }
+  else {
+    // Assume that a vertical view has limited space, thus the number of boxes
+    // is hardcoded
+    total_no_boxes = 5;
+    box_spacing = 2;
+    int const total_no_spacings = total_no_boxes - 1;
+    box_size = int(double(height - (total_no_spacings * box_spacing)) / total_no_boxes);
+  }
+  
+  
   // don't attain new value immediately
   double value = old_value * (1 - time_offset) + new_value * time_offset;
 
@@ -144,7 +162,7 @@ void Bar::draw(Gnome::Canvas::Canvas &canvas,
     boxes.pop_back();
   }
 
-  double x = 0;
+  double coord = this->horizontal ? 0 : height;
   // update parameters, starting from left
   for (box_sequence::iterator i = boxes.begin(), end = boxes.end(); i != end;
        ++i) {
@@ -152,12 +170,22 @@ void Bar::draw(Gnome::Canvas::Canvas &canvas,
     rect.property_fill_color_rgba() = fill_color;
     rect.property_outline_color_rgba() = outline_color;
     
-    rect.property_x1() = x;
-    rect.property_x2() = x + box_width;
-    rect.property_y1() = double(height) * no / total + 1;
-    rect.property_y2() = double(height) * (no + 1) / total - 1;
-
-    x += box_width + box_spacing;
+    if (this->horizontal) {
+      rect.property_x1() = coord;
+      rect.property_x2() = coord + box_size;
+      rect.property_y1() = double(height) * no / total + 1;
+      rect.property_y2() = double(height) * (no + 1) / total - 1;
+      
+      coord += box_size + box_spacing;
+    }
+    else {
+	    rect.property_x1() = double(width) * no / total + 1;
+	    rect.property_x2() = double(width) * (no + 1) / total - 1;
+	    rect.property_y1() = coord;
+	    rect.property_y2() = coord - box_size;
+      
+      coord -= (box_size + box_spacing);
+    }
   }
 
   // tint last box
@@ -177,9 +205,10 @@ void Bar::draw(Gnome::Canvas::Canvas &canvas,
 // class BarView
 //
 
-BarView::BarView()
+BarView::BarView(bool _horizontal)
   : CanvasView(false), draws_since_update(0)
 {
+  horizontal = _horizontal;
 }
 
 BarView::~BarView()
@@ -200,7 +229,7 @@ void BarView::do_update()
 
 void BarView::do_attach(Monitor *monitor)
 {
-  bars.push_back(new Bar(monitor));
+  bars.push_back(new Bar(monitor, this->horizontal));
 }
 
 void BarView::do_detach(Monitor *monitor)
@@ -228,3 +257,6 @@ void BarView::do_draw_loop()
   ++draws_since_update;
 }
 
+bool BarView::is_horizontal() {
+  return this->horizontal;
+}
