@@ -1,7 +1,7 @@
 /* A view which displays steady flames, the sizes of which are determined by the
  * monitor values.
  *
- * Copyright (c) 2003 Ole Laursen.
+ * Copyright (c) 2003, 04 Ole Laursen.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -42,7 +42,7 @@ public:
   
 private:
   virtual void do_update();
-  virtual void do_attach(Monitor *monitor, const Glib::ustring &mon_dir);
+  virtual void do_attach(Monitor *monitor);
   virtual void do_detach(Monitor *monitor);
   virtual void do_draw_loop();
 
