@@ -23,9 +23,11 @@
 #include <unistd.h>		// for nice
 #include <libintl.h>
 
-#include <libgnomemm/main.h>
-#include <libgnomeuimm/init.h>
+#include <libgnomecanvasmm/init.h>
+#include <gconfmm/init.h>
+#include <gtkmm/main.h>
 
+#include <libgnomeui/gnome-ui-init.h>
 #include <panel-applet.h>
 
 #include "applet.hpp"
@@ -48,9 +50,17 @@ gboolean hardware_monitor_factory(PanelApplet *panel_applet, const gchar *iid,
 int main(int argc, char *argv[])
 {
   nice(5);			// don't eat up too much CPU
-  
-  Gnome::Main main(PACKAGE, VERSION, Gnome::UI::module_info_get(), argc, argv);
-  
+
+  Gtk::Main main(argc, argv);
+  Gnome::Conf::init();
+  Gnome::Canvas::init();
+
+  // this is necessary for panel applets
+  gnome_program_init(PACKAGE, VERSION,
+		     LIBGNOMEUI_MODULE,
+		     argc, argv, GNOME_CLIENT_PARAM_SM_CONNECT, FALSE,
+		     NULL);
+		   
   try {
     // i18n
     bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
