@@ -233,16 +233,17 @@ Precision decimal_digits(double val, int n)
 // class CpuUsageMonitor
 //
 
-int const CpuUsageMonitor::max_cpu_no = GLIBTOP_NCPU;
+int const CpuUsageMonitor::max_no_cpus = GLIBTOP_NCPU;
 
 CpuUsageMonitor::CpuUsageMonitor()
-  : cpu_no(all_cpus), user_time(0), nice_time(0), sys_time(0), idle_time(0)
+  : cpu_no(all_cpus), total_time(0), nice_time(0), idle_time(0)
 {}
 
 CpuUsageMonitor::CpuUsageMonitor(int cpu)
-  : cpu_no(cpu), user_time(0), nice_time(0), sys_time(0), idle_time(0)
+  : cpu_no(cpu), total_time(0), nice_time(0), idle_time(0)
 {
-  assert(cpu_no >= 0 && cpu_no < GLIBTOP_NCPU);
+  if (cpu_no < 0 || cpu_no >= max_no_cpus)
+    cpu_no = all_cpus;
 }
 
 double CpuUsageMonitor::do_measure()
@@ -251,40 +252,36 @@ double CpuUsageMonitor::do_measure()
 	
   glibtop_get_cpu(&cpu);
 
-  guint64 u, n, s, i;
+  guint64 t, n, i;
 	
   if (cpu_no == all_cpus) {
-    u = cpu.user;
+    t = cpu.total;
     n = cpu.nice;
-    s = cpu.sys;
     i = cpu.idle;
   }
   else {
-    u = cpu.xcpu_user[cpu_no];
+    t = cpu.xcpu_total[cpu_no];
     n = cpu.xcpu_nice[cpu_no];
-    s = cpu.xcpu_sys[cpu_no];
     i = cpu.xcpu_idle[cpu_no];
   }
 	
   // calculate ticks since last call
   guint64
-    duser = u - user_time,
+    dtotal = t - total_time,
     dnice = n - nice_time,
-    dsys = s - sys_time,
     didle = i - idle_time; 
 
   // and save the new values
-  user_time = u;
+  total_time = t;
   nice_time = n;
-  sys_time = s;
   idle_time = i;
-    
-  guint64 total = duser + dnice + dsys + didle;
 
-  if (total > 0)
-    // don't count dnice to avoid always showing 100% with SETI@home and
-    // similar applications running
-    return double(duser + dsys) / total;
+  // don't count in dnice to avoid always showing 100% with SETI@home and
+  // similar applications running
+  double res = double(dtotal - dnice - didle) / dtotal;
+
+  if (res > 0)
+    return res;
   else
     return 0;
 }
@@ -562,18 +559,18 @@ Glib::ustring DiskUsageMonitor::format_value(double val)
 {
   if (val >= 1000 * 1000 * 1000) {
     val /= 1000 * 1000 * 1000;
-    return String::ucompose(_("%1 Gb"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 GB"), decimal_digits(val, 3), val);
   }
   else if (val >= 1000 * 1000) {
     val /= 1000 * 1000;
-    return String::ucompose(_("%1 Mb"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 MB"), decimal_digits(val, 3), val);
   }
   else if (val >= 1000) {
     val /= 1000;
-    return String::ucompose(_("%1 kb"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 kB"), decimal_digits(val, 3), val);
   }
   else
-    return String::ucompose(_("%1 b"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 B"), decimal_digits(val, 3), val);
 }
 
 Glib::ustring DiskUsageMonitor::get_name()
@@ -618,23 +615,25 @@ double NetworkLoadMonitor::do_measure()
 
   glibtop_get_netload(&netload,
 		      String::ucompose("%1%2", interface, interface_no).c_str());
-  guint64 val, bytes;
+  guint64 val, measured_bytes;
     
   if (direction == all_data)
-    bytes = netload.bytes_total;
+    measured_bytes = netload.bytes_total;
   else if (direction == incoming_data)
-    bytes = netload.bytes_in;
+    measured_bytes = netload.bytes_in;
   else
-    bytes = netload.bytes_out;
-  
-  if (byte_count == 0)
-    val = 0;   // avoid calculating an enormous throughput initially
+    measured_bytes = netload.bytes_out;
+
+  if (byte_count == 0) // no estimate initially
+    val = 0;
+  else if (measured_bytes < byte_count)	// interface was reset
+    val = 0;
   else
-    val = bytes - byte_count;
+    val = measured_bytes - byte_count;
 
-  byte_count = bytes;
+  byte_count = measured_bytes;
 
-  if (val != 0)			// reduce gradually
+  if (val != 0)			// reduce scale gradually
     max_value = guint64(max_value * max_decay);
   
   if (val > max_value)
@@ -677,18 +676,18 @@ Glib::ustring NetworkLoadMonitor::format_value(double val)
   
   if (val >= 1000 * 1000 * 1000) {
     val /= 1000 * 1000 * 1000;
-    return String::ucompose(_("%1 Gb/s"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 GB/s"), decimal_digits(val, 3), val);
   }
   else if (val >= 1000 * 1000) {
     val /= 1000 * 1000;
-    return String::ucompose(_("%1 Mb/s"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 MB/s"), decimal_digits(val, 3), val);
   }
   else if (val >= 1000) {
     val /= 1000;
-    return String::ucompose(_("%1 kb/s"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 kB/s"), decimal_digits(val, 3), val);
   }
   else 
-    return String::ucompose(_("%1 b/s"), decimal_digits(val, 3), val);
+    return String::ucompose(_("%1 B/s"), decimal_digits(val, 3), val);
 }
 
 Glib::ustring NetworkLoadMonitor::get_name()
