@@ -35,6 +35,13 @@
 #include "i18n.hpp"
 
 
+void PreferencesWindow::connect_monitor_colorbutton(Gtk::ColorButton
+						    *colorbutton)
+{
+  colorbutton->signal_color_set()
+    .connect(sigc::bind(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorbutton_set),
+			colorbutton));
+}
 
 PreferencesWindow::PreferencesWindow(Applet &applet_, monitor_seq monitors)
   : applet(applet_)
@@ -57,6 +64,11 @@ PreferencesWindow::PreferencesWindow(Applet &applet_, monitor_seq monitors)
     .connect(sigc::mem_fun(*this, &PreferencesWindow::
 			on_bar_radiobutton_toggled));
   
+  ui->get_widget("vbar_radiobutton", vbar_radiobutton);
+  vbar_radiobutton->signal_toggled()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
+			on_vbar_radiobutton_toggled));
+  
   ui->get_widget("column_radiobutton", column_radiobutton);
   column_radiobutton->signal_toggled()
     .connect(sigc::mem_fun(*this, &PreferencesWindow::
@@ -85,15 +97,15 @@ PreferencesWindow::PreferencesWindow(Applet &applet_, monitor_seq monitors)
     .connect(sigc::mem_fun(*this, &PreferencesWindow::
 			on_font_checkbutton_toggled));
 
-  ui->get_widget("fontpicker", fontpicker);
-  fontpicker->signal_font_set()
-    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_fontpicker_set));
+  ui->get_widget("fontbutton", fontbutton);
+  fontbutton->signal_font_set()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_fontbutton_set));
 
 
-  ui->get_widget("background_colorpicker", background_colorpicker);
-  background_colorpicker->signal_color_set()
+  ui->get_widget("background_colorbutton", background_colorbutton);
+  background_colorbutton->signal_color_set()
     .connect(sigc::mem_fun(*this, &PreferencesWindow::
-			on_background_colorpicker_set));
+			on_background_colorbutton_set));
 
   ui->get_widget("panel_background_radiobutton", panel_background_radiobutton);
   ui->get_widget("background_color_radiobutton", background_color_radiobutton);
@@ -129,24 +141,24 @@ PreferencesWindow::PreferencesWindow(Applet &applet_, monitor_seq monitors)
   monitor_treeview->append_column(_("Device"), mc.name);
   
   ui->get_widget("monitor_curve_options", monitor_curve_options);
-  ui->get_widget("line_colorpicker", line_colorpicker);
-  line_colorpicker->signal_color_set()
-    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
-
+  ui->get_widget("line_colorbutton", line_colorbutton);
+  connect_monitor_colorbutton(line_colorbutton);
+  
   ui->get_widget("monitor_bar_options", monitor_bar_options);
-  ui->get_widget("bar_colorpicker", bar_colorpicker);
-  bar_colorpicker->signal_color_set()
-    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
+  ui->get_widget("bar_colorbutton", bar_colorbutton);
+  connect_monitor_colorbutton(bar_colorbutton);
+  
+  ui->get_widget("monitor_vbar_options", monitor_vbar_options);
+  ui->get_widget("vbar_colorbutton", vbar_colorbutton);
+  connect_monitor_colorbutton(vbar_colorbutton);
   
   ui->get_widget("monitor_column_options", monitor_column_options);
-  ui->get_widget("column_colorpicker", column_colorpicker);
-  column_colorpicker->signal_color_set()
-    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
+  ui->get_widget("column_colorbutton", column_colorbutton);
+  connect_monitor_colorbutton(column_colorbutton);
 
   ui->get_widget("monitor_flame_options", monitor_flame_options);
-  ui->get_widget("flame_colorpicker", flame_colorpicker);
-  flame_colorpicker->signal_color_set()
-    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
+  ui->get_widget("flame_colorbutton", flame_colorbutton);
+  connect_monitor_colorbutton(flame_colorbutton);
   
   
   // connect GConf
@@ -183,7 +195,7 @@ PreferencesWindow::PreferencesWindow(Applet &applet_, monitor_seq monitors)
   // between the greyed-out buttons and the treeview
   monitor_treeview->get_selection()->unselect_all();
 
-  // make sure background colorpicker is grayed out
+  // make sure background colorbutton is grayed out
   background_color_radiobutton->toggled();
   
   // connect close operations
@@ -210,6 +222,34 @@ void PreferencesWindow::show()
 }
 
 
+namespace 
+{
+  void update_colorbutton_if_different(Gtk::ColorButton *colorbutton,
+				       unsigned char r,
+				       unsigned char g,
+				       unsigned char b,
+				       unsigned char a)
+  {
+    unsigned char pa, pr, pg, pb;
+    
+    pa = colorbutton->get_alpha() >> 8;
+  
+    Gdk::Color c = colorbutton->get_color();
+    pr = c.get_red() >> 8;
+    pg = c.get_green() >> 8;
+    pb = c.get_blue() >> 8;
+    
+    if (pr != r || pg != g || pb != b) {
+      Gdk::Color new_c;
+      new_c.set_rgb(gushort(r) << 8, gushort(g) << 8, gushort(b) << 8);
+      colorbutton->set_color(new_c);
+    }
+    else if (pa != a)
+      colorbutton->set_alpha(gushort(a) << 8);
+  }
+}
+
+
 // GConf callbacks
 
 void PreferencesWindow::viewer_type_listener(unsigned int,
@@ -232,6 +272,12 @@ void PreferencesWindow::viewer_type_listener(unsigned int,
     size_outer_vbox->property_visible() = true;
     monitor_bar_options->property_visible() = true;
   }
+  else if (s == "vbar") {
+    if (!vbar_radiobutton->get_active())
+      vbar_radiobutton->property_active() = true;
+    size_outer_vbox->property_visible() = true;
+    monitor_vbar_options->property_visible() = true;
+  }
   else if (s == "column") {
     if (!column_radiobutton->get_active())
       column_radiobutton->property_active() = true;
@@ -260,11 +306,8 @@ void PreferencesWindow::background_color_listener(unsigned int,
   unsigned int i = entry.get_value().get_int();
   
   unsigned char r = i >> 24, g = i >> 16, b = i >> 8, a = i;
-  unsigned char pr, pg, pb, pa;
 
-  background_colorpicker->get(pr, pg, pb, pa);
-  if (pr != r || pg != g || pb != b || pa != a)
-    background_colorpicker->set(r, g, b, a);
+  update_colorbutton_if_different(background_colorbutton, r, g, b, a);
 }
 
 void PreferencesWindow::use_background_color_listener(unsigned int,
@@ -304,8 +347,8 @@ void PreferencesWindow::font_listener(unsigned int, Gnome::Conf::Entry entry)
     font_checkbutton->set_active(false);
   else {
     font_checkbutton->set_active(true);
-    if (fontpicker->get_font_name() != font)
-      fontpicker->set_font_name(font);
+    if (fontbutton->get_font_name() != font)
+      fontbutton->set_font_name(font);
   }
 }
 
@@ -318,46 +361,52 @@ void PreferencesWindow::monitor_color_listener(unsigned int,
   unsigned int i = entry.get_value().get_int();
 
   unsigned char r = i >> 24, g = i >> 16, b = i >> 8, a = i;
-  unsigned char pr, pg, pb, pa;
 
-  line_colorpicker->get(pr, pg, pb, pa);
-  if (pr != r || pg != g || pb != b || pa != a)
-    line_colorpicker->set(r, g, b, a);
-  
-  bar_colorpicker->get(pr, pg, pb, pa);
-  if (pr != r || pg != g || pb != b || pa != a)
-    bar_colorpicker->set(r, g, b, a);
-
-  column_colorpicker->get(pr, pg, pb, pa);
-  if (pr != r || pg != g || pb != b || pa != a)
-    column_colorpicker->set(r, g, b, a);
-  
-  flame_colorpicker->get(pr, pg, pb, pa);
-  if (pr != r || pg != g || pb != b || pa != a)
-    flame_colorpicker->set(r, g, b, a);
+  update_colorbutton_if_different(line_colorbutton, r, g, b, a);
+  update_colorbutton_if_different(bar_colorbutton,  r, g, b, a);
+  update_colorbutton_if_different(vbar_colorbutton,  r, g, b, a);
+  update_colorbutton_if_different(column_colorbutton, r, g, b, a);
+  update_colorbutton_if_different(flame_colorbutton,  r, g, b, a);
 }
 
 
-
-
 // UI callbacks
 
-unsigned int to_int(unsigned int r, unsigned int g, unsigned int b,
-		    unsigned int a)
+namespace 
 {
-  return ((r & 255) << 24) | ((g & 255) << 16) | ((b & 255) << 8) | (a & 255);
+  // helper for avoiding clipping when shifting values
+  unsigned int pack_int(unsigned int r, unsigned int g, unsigned int b,
+		      unsigned int a)
+  {
+    return ((r & 255) << 24) | ((g & 255) << 16) | ((b & 255) << 8) | (a & 255);
+  }
+
 }
 
-void PreferencesWindow::on_background_colorpicker_set(unsigned int r,
-						      unsigned int g,
-						      unsigned int b,
-						      unsigned int a)
+void PreferencesWindow::sync_conf_with_colorbutton(std::string path,
+						   Gtk::ColorButton *button)
 {
+  // extract info from button
+  unsigned char a, r, g, b;
+    
+  a = button->get_alpha() >> 8;
+  
+  Gdk::Color c = button->get_color();
+  r = c.get_red() >> 8;
+  g = c.get_green() >> 8;
+  b = c.get_blue() >> 8;
+  
+  // update configuration
   Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
-  Glib::ustring dir = applet.get_gconf_dir();
 
-  client->set(dir + "/background_color", int(to_int(r / 256, g / 256,
-						    b / 256, a / 256)));
+  client->set(path, int(pack_int(r, g, b, a)));
+}
+
+
+void PreferencesWindow::on_background_colorbutton_set()
+{
+  sync_conf_with_colorbutton(applet.get_gconf_dir() + "/background_color",
+			     background_colorbutton);
 }
 
 void PreferencesWindow::on_background_color_radiobutton_toggled()
@@ -367,7 +416,7 @@ void PreferencesWindow::on_background_color_radiobutton_toggled()
 
   bool on = background_color_radiobutton->get_active();
   
-  background_colorpicker->set_sensitive(on);
+  background_colorbutton->set_sensitive(on);
   
   client->set(dir + "/use_background_color", on);
 }
@@ -400,6 +449,20 @@ void PreferencesWindow::on_bar_radiobutton_toggled()
   monitor_bar_options->property_visible() = active;
 }
 
+void PreferencesWindow::on_vbar_radiobutton_toggled()
+{
+  Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
+  Glib::ustring dir = applet.get_gconf_dir();
+
+  bool active = vbar_radiobutton->get_active();
+  
+  if (active)
+    client->set(dir + "/viewer_type", Glib::ustring("vbar"));
+
+  size_outer_vbox->property_visible() = active;
+  monitor_vbar_options->property_visible() = active;
+}
+
 void PreferencesWindow::on_column_radiobutton_toggled()
 {
   Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
@@ -460,20 +523,20 @@ void PreferencesWindow::on_font_checkbutton_toggled()
 
   bool active = font_checkbutton->get_active();
   
-  fontpicker->set_sensitive(active);
+  fontbutton->set_sensitive(active);
 
   if (active)
-    client->set(dir + "/viewer/font", fontpicker->get_font_name());
+    client->set(dir + "/viewer/font", fontbutton->get_font_name());
   else
     client->set(dir + "/viewer/font", Glib::ustring(""));
 }
 
-void PreferencesWindow::on_fontpicker_set(const Glib::ustring &font)
+void PreferencesWindow::on_fontbutton_set()
 {
   Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
   Glib::ustring dir = applet.get_gconf_dir();
-  
-  client->set(dir + "/viewer/font", font);
+
+  client->set(dir + "/viewer/font", fontbutton->get_font_name());
 }
 
 void PreferencesWindow::on_add_button_clicked()
@@ -559,21 +622,16 @@ void PreferencesWindow::on_selection_changed()
   monitor_options->set_sensitive(sel);
 }
 
-void PreferencesWindow::on_monitor_colorpicker_set(unsigned int r,
-						   unsigned int g,
-						   unsigned int b,
-						   unsigned int a)
+void PreferencesWindow::on_monitor_colorbutton_set(Gtk::ColorButton *colorbutton)
 {
   static MonitorColumns mc;
   
-  Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
-  
   store_iter i = monitor_treeview->get_selection()->get_selected();
   
   if (i) {
     Glib::ustring mon_dir = (*(*i)[mc.monitor]).get_gconf_dir();
-    client->set(mon_dir + "/color", int(to_int(r / 256, g / 256,
-					       b / 256, a / 256)));
+
+    sync_conf_with_colorbutton(mon_dir + "/color", colorbutton);
   }
 }
 
@@ -623,7 +681,7 @@ int PreferencesWindow::size_scale_to_pixels(int size)
 int PreferencesWindow::pixels_to_size_scale(int pixels)
 {
   // we may not have an exact match, so just find the nearest
-  int min_diff, min_i;
+  int min_diff = 1000000, min_i = 0;
   for (int i = 0; i < pixel_size_table_size; ++i) {
     int diff = std::abs(pixel_size_table[i] - pixels);
     if (diff < min_diff) {
@@ -634,4 +692,3 @@ int PreferencesWindow::pixels_to_size_scale(int pixels)
 
   return min_i;
 }
-
